@@ -1,6 +1,9 @@
+__license__ = "GNU General Public License v3"
+__author__ = 'Hwaipy'
+__email__ = 'hwaipy@gmail.com'
+
 import zmq
 from zmq.eventloop.zmqstream import ZMQStream
-from tornado.ioloop import IOLoop
 from interactionfreepy.core import IFException, Message, Invocation, IFLoop, IFDefinition
 import time
 import threading
@@ -11,7 +14,7 @@ class IFWorker(object):
     def __init__(self, endpoint, serviceName=None, serviceObject=None, interfaces=[], blocking=True, timeout=None, force=False):
         self.__endpoint = endpoint
         self.socket = zmq.Context().socket(zmq.DEALER)
-        self.__stream = ZMQStream(self.socket, IOLoop.current())
+        self.__stream = ZMQStream(self.socket, IFLoop.INSTANCE)
         self.__stream.on_recv(self.__onMessage)
         self.__stream.socket.setsockopt(zmq.LINGER, 0)
         self.__stream.connect(self.__endpoint)
@@ -41,7 +44,7 @@ class IFWorker(object):
             message = Message(msg)
             invocation = message.getInvocation()
             if invocation.isRequest():
-                IOLoop.current().add_callback(self.__onRequest, message)
+                IFLoop.INSTANCE.add_callback(self.__onRequest, message)
             elif invocation.isResponse():
                 self.__onResponse(message)
         except BaseException as e:
@@ -107,7 +110,7 @@ class IFWorker(object):
 
     @classmethod
     def start(cls):
-        IOLoop.instance().start()
+        IFLoop.INSTANCE.start()
 
     def __getattr__(self, item):
         return InvokeTarget(self, item)
@@ -305,6 +308,7 @@ class InvokeFuture:
 
 if __name__ == '__main__':
     pass
+
     # class Target:
     #     def test(self, arg):
     #         print(arg)
